<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
	xmlns="http://www.w3.org/1999/xhtml"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:seaf="http://seaf.sf.net/seaf"
	xmlns:per="http://seaf.sf.net/seaf-persistence"
	xmlns:svc="http://seaf.sf.net/seaf-service"
	version="2.0">
<xsl:import href="functions/SeafServiceBusinessFunctions.xsl" />
<xsl:import href="generator/seaf-persistence/Report.xsl" />

<xsl:param name="application-name" />
<xsl:param name="version" />
<xsl:param name="persistence-merged-file-name" />

<xsl:variable name="merged-persistence-file" select="document($persistence-merged-file-name)" />
<xsl:variable name="merged-service-file" select="/" />

<xsl:variable name="entities" select="$merged-persistence-file//per:entity" />
<xsl:variable name="relations" select="$merged-persistence-file//per:relation" />
<xsl:variable name="objects" select="$merged-service-file//svc:object" />

<xsl:variable name="title">
	<xsl:text />Service interface <xsl:value-of select="$application-name"/>
	<xsl:text /> v. <xsl:value-of select="$version"/>
</xsl:variable>

<xsl:template match="/">
	<html>
		<head>
			<title><xsl:value-of select="$title"/></title>
			<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
			<style type="text/css">
.label {font-weight:bold;}
table {border-spacing:2; border-collapse:collapse;}
table, td {border:1px solid black;}
td {padding:2 4;}
ul {margin-top: 0px; margin-bottom: 0px;}
ol {margin-top: 0px; margin-bottom: 0px;}
			</style>
		</head>
	<body>
		<h1><xsl:value-of select="$title"/></h1>
		<xsl:call-template name="module-list-template" />
		<h2>List of service interfaces</h2>
		<xsl:call-template name="interface-list-template">
			<xsl:with-param name="nodes" select="." />
		</xsl:call-template>
		<h2>List of entities</h2>
		<xsl:call-template name="entity-list-template">
			<xsl:with-param name="nodes" select="." />
		</xsl:call-template>
		<xsl:call-template name="modules-template" />
		<xsl:apply-templates select="//svc:service" />
		<xsl:apply-templates select="//svc:service//svc:operation" />
		<xsl:apply-templates select="
			for $name in distinct-values(.//svc:object/@entity | .//svc:enum-def/@entity)
			return $merged-persistence-file//per:entity[@name=$name] | 
				$merged-persistence-file//per:enum-def[@name=$name]" />
	</body>
	</html>
</xsl:template>

<xsl:template name="module-list-template">
	<h2>List of modules</h2>
	<table>
		<tr><td>Name</td></tr>
	<xsl:for-each select="distinct-values(//seaf:module/@name)">
		<tr><td><xsl:copy-of select="svc:link('module',.)" /></td></tr>
	</xsl:for-each>
	</table>
</xsl:template>

<xsl:template name="modules-template">
	<xsl:for-each select="distinct-values(//seaf:module/@name)">
		<xsl:call-template name="module-template">
			<xsl:with-param name="name" select="." />
		</xsl:call-template>
	</xsl:for-each>
</xsl:template>

<xsl:template name="module-template">
<xsl:param name="name" as="xs:string"  />
	<xsl:variable name="modules" select="$merged-service-file//seaf:module[@name=$name]" />
	<xsl:variable name="module" select="$modules[1]" />
	<xsl:text>
	</xsl:text>
	<a name="module_{$name}"><h2>Module: <xsl:value-of select="$name" /></h2></a>
	<span class="label">Package:</span><xsl:text> </xsl:text>
		<xsl:value-of select="$module/@package" /><br />
	<xsl:text>
	</xsl:text>
	<span class="label">Module interfaces:</span>
	<xsl:call-template name="interface-list-template">
		<xsl:with-param name="nodes" select="$modules" />
	</xsl:call-template>
	<span class="label">Module entities:</span>
	<xsl:call-template name="entity-list-template">
		<xsl:with-param name="nodes" select="$modules" />
	</xsl:call-template>
</xsl:template>

<xsl:template name="interface-list-template">
<xsl:param name="nodes" as="node()*"  />
	<xsl:choose>
	<xsl:when test="exists($nodes//svc:service)">
	<table>
		<tr><td>Name</td></tr>
	<xsl:for-each select="$nodes//svc:service">
		<tr>
			<td><xsl:copy-of select="svc:link('interface',@interface)" /></td>
		</tr>
	</xsl:for-each>
	</table>
	</xsl:when>
	<xsl:otherwise>
		<br />
		<xsl:text>
	</xsl:text>
	</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="entity-list-template">
<xsl:param name="nodes" as="node()*"  />
	<table>
		<tr><td>Name</td><td>Type</td><td>Description</td></tr>
	<xsl:for-each select="
		for $name in distinct-values($nodes//svc:object/@entity | $nodes//svc:enum-def/@entity)
		return $merged-persistence-file//per:entity[@name=$name] | 
			$merged-persistence-file//per:enum-def[@name=$name]">
		<tr>
			<td><xsl:copy-of select="svc:link('entity',@name)" /></td>
			<td><xsl:call-template name="entity-type-template" /></td>
			<td><xsl:value-of select="per:description[1]"/></td>
		</tr>
	</xsl:for-each>
	</table>
</xsl:template>

<xsl:template match="svc:service">
	<xsl:text>
	</xsl:text>
	<a name="interface_{@interface}">
		<h2>Service interface: <xsl:value-of select="@interface" /></h2></a>
	<xsl:for-each select="svc:controller">
	<xsl:text>
	</xsl:text>
	<span class="label">Operations on controller of entity <xsl:text />
		<xsl:value-of select="@entity" />:</span>
	<table>
		<tr><td>Name</td><td>Short description</td></tr>
		<xsl:for-each select="svc:operation">
		<tr>
			<td><xsl:copy-of select="svc:link('operation',@name)" /></td>
			<td><xsl:value-of select="svc:description" /></td>
		</tr>
		</xsl:for-each>
	</table>
	</xsl:for-each>
</xsl:template>

<xsl:template match="svc:operation">
	<xsl:variable name="request">
		<xsl:choose>
		<xsl:when test="svc:request-void">void</xsl:when>
		<xsl:otherwise>
			<xsl:variable name="object-full-type" 
				select="svc:ref-service-object-full-type(svc:request-object,$objects)" />
			<xsl:copy-of select="svc:link('object',seaf:type-class($object-full-type))" />
		</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:variable name="response">
		<xsl:choose>
		<xsl:when test="svc:response-void">void</xsl:when>
		<xsl:otherwise>
			<xsl:variable name="object-full-type" select="svc:ref-service-object-full-type(
				svc:response-object|svc:response-sequence,$objects)" />
			<xsl:choose>
			<xsl:when test="svc:response-sequence/@paged='true'">Paged array [</xsl:when>
			<xsl:when test="svc:response-sequence">Array [</xsl:when>
			</xsl:choose>
			<xsl:copy-of select="svc:link('object',seaf:type-class($object-full-type))" />
			<xsl:if test="svc:response-sequence">]</xsl:if>
		</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:text>
	</xsl:text>
	<a name="operation_{@name}"><h2>Operation: <xsl:value-of select="@name" /></h2></a>
	<table>
	<tr>
	<td><span class="label">Module:</span><xsl:text> </xsl:text>
		<xsl:copy-of select="svc:link('module',ancestor::seaf:module/@name)" /></td>
	<td><span class="label">Controller of entity:</span><xsl:text> </xsl:text>
		<xsl:copy-of select="svc:link('entity',ancestor::svc:controller/@entity)" /></td>
	</tr>
	<tr>
	<td><span class="label">Service interface:</span><xsl:text> </xsl:text>
		<xsl:copy-of select="svc:link('interface',ancestor::svc:service/@interface)" /></td>
	<td><xsl:if test="@method">
		<span class="label">Controller method:</span><xsl:text> </xsl:text>
		<xsl:value-of select="@method" />
		</xsl:if></td>
	</tr>
	<tr>
	<td><span class="label">Request:</span><xsl:text> </xsl:text>
		<xsl:copy-of select="$request" />
		<xsl:if test="svc:request-object/@pass-as='service-object'">
			<xsl:text> passed as service object</xsl:text>
		</xsl:if>
	</td>
	<td><span class="label">Response:</span><xsl:text> </xsl:text>
		<xsl:copy-of select="$response" /></td>
	</tr>
	<xsl:if test="svc:throws">
	<tr>
	<td colspan="2">
		<span class="label">Throws:</span><xsl:text> </xsl:text>
		<xsl:for-each select="svc:throws">
			<xsl:value-of select="@class" />
			<xsl:if test="not(position()=last())">, </xsl:if>
		</xsl:for-each>
	</td>
	</tr>
	</xsl:if>
	<xsl:if test="svc:role">
	<tr>
	<td colspan="2">
		<span class="label">Roles:</span><xsl:text> </xsl:text>
		<xsl:for-each select="svc:role">
			<xsl:value-of select="@name" />
			<xsl:if test="not(position()=last())">, </xsl:if>
		</xsl:for-each>
	</td>
	</tr>
	</xsl:if>
	<tr>
	<td colspan="2">
		<b><xsl:call-template name="svc-descriptions-template" /></b>
		<xsl:apply-templates select="svc:description-list" />
	</td>
	</tr>
	</table>
</xsl:template>

<xsl:template match="svc:description-list">
	<xsl:choose>
		<xsl:when test="@type='un-ordered'">
		<ul><xsl:call-template name="description-list-content-template" /></ul>
		</xsl:when>
		<xsl:otherwise>
		<ol><xsl:call-template name="description-list-content-template" /></ol>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="description-list-content-template">
	<xsl:for-each select="svc:description | svc:description-list">
		<xsl:choose>
			<xsl:when test="local-name(.)='description-list'">
				<xsl:apply-templates select="."/>
			</xsl:when>
			<xsl:otherwise>
				<li><xsl:value-of select="."/></li>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:for-each>
</xsl:template>

<xsl:template match="per:entity | per:enum-def">
	<xsl:variable name="entity" select="@name" />
	<xsl:text>
	</xsl:text>
	<a name="entity_{@name}"><h2>Entity: <xsl:value-of select="@name" /></h2></a>
	<span class="label">Module:</span><xsl:text> </xsl:text>
		<xsl:copy-of select="svc:link('module',ancestor::seaf:module/@name)" /><br />
	<span class="label">Type:</span><xsl:text> </xsl:text>
		<xsl:call-template name="entity-type-template" /><br />
	<span class="label">Description:</span><xsl:text> </xsl:text>
		<xsl:call-template name="per-descriptions-template" /><br />
<xsl:if test="local-name()='entity'">
	<xsl:variable name="fields" select="per:entity-fields(.,())" />
	<span class="label">Fields:</span>
	<table>
		<tr><td>Name</td><td>Status</td><td>Type</td><td>Description</td></tr>
	<xsl:apply-templates select="$fields" />
	<xsl:for-each select="$merged-persistence-file//per:relation[per:detail/@entity-name=$entity]">
		<xsl:variable name="status" select="per:master-status(per:master/@status)" />
		<tr>
			<td><xsl:value-of select="per:relation-property-name(per:detail,.)"/></td>
			<td title="{per:status-hint($status)}">
				<xsl:value-of select="$status" />
			</td>
			<td><xsl:copy-of select="svc:link('entity',per:master/@entity-name)"/></td>
			<td>Relation to master entity.</td>
		</tr>
	</xsl:for-each>
	<xsl:for-each select="$merged-persistence-file//per:relation[per:master/@entity-name=$entity]">
		<tr>
			<td><xsl:value-of select="per:relation-property-name(per:master,.)"/></td>
			<td>O</td>
			<td>
				<xsl:if test="@type='one-to-many'">List [</xsl:if>
				<xsl:copy-of select="svc:link('entity',per:detail/@entity-name)"/>
				<xsl:if test="@type='one-to-many'">]</xsl:if>
			</td>
			<td>Relation to detail entity.</td>
		</tr>
	</xsl:for-each>
	</table>
	<xsl:if test="@code-table">
		<xsl:call-template name="references-template">
			<xsl:with-param name="entities" 
				select="$merged-persistence-file//per:entity[per:field/per:code-table/@name=$entity]" />
		</xsl:call-template>
	</xsl:if>
</xsl:if>
<xsl:if test="local-name()='enum-def'">
	<span class="label">Values:</span>
	<table>
		<tr><td>Code</td><td>Description</td></tr>
	<xsl:for-each select=".//per:enum-value">
		<tr>
			<td><xsl:value-of select="@code"/></td>
			<td><xsl:value-of select="@description"/></td>
		</tr>
	</xsl:for-each>
	</table>
	<xsl:call-template name="references-template">
		<xsl:with-param name="entities" 
			select="$merged-persistence-file//per:entity[per:field/per:enum/@name=$entity]" />
	</xsl:call-template>
</xsl:if>
	<xsl:apply-templates select="$merged-service-file//svc:object[@entity=$entity] | 
		$merged-service-file//svc:enum-def[@entity=$entity]" />
</xsl:template>

<xsl:template match="per:field">
	<xsl:variable name="status" select="per:field-status(@status)" />
	<tr>
		<td><xsl:value-of select="@name"/></td>
		<td title="{per:status-hint($status)}">
			<xsl:value-of select="$status" />
		</td>
		<td><xsl:call-template name="field-type-template" /></td>
		<td><xsl:call-template name="per-descriptions-template" /></td>
	</tr>
</xsl:template>

<xsl:template name="references-template">
<xsl:param name="entities" as="node()*" />
	<span class="label">References:</span>
	<xsl:for-each select="$entities">
		<xsl:text> </xsl:text>
		<xsl:copy-of select="svc:link('entity',@name)"/>
		<xsl:if test="not(position()=last())"><xsl:text>,</xsl:text></xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="per-descriptions-template">
	<xsl:for-each select="per:description">
		<xsl:value-of select="." />
		<xsl:if test="not(position()=last())"><br /></xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="svc-descriptions-template">
	<xsl:for-each select="svc:description">
		<xsl:value-of select="." />
		<xsl:if test="not(position()=last())"><br /></xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="entity-type-template">
	<xsl:choose>
		<xsl:when test="local-name()='enum-def'">Enumeration</xsl:when>
		<xsl:when test="@code-table='code-based'">Code-based code table</xsl:when>
		<xsl:when test="@code-table='time-bound'">Time-bound code table</xsl:when>
		<xsl:otherwise>Entity</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:function name="per:master-status" as="xs:string">
<xsl:param name="status" />
	<xsl:choose>
		<xsl:when test="not($status='optional')">M</xsl:when>
		<xsl:otherwise>O</xsl:otherwise>
	</xsl:choose>
</xsl:function>

<xsl:template name="field-type-template">
	<xsl:choose>
		<xsl:when test="per:string">
			<xsl:text>String [</xsl:text>
			<xsl:value-of select="per:string/@length" />
			<xsl:text>]</xsl:text>
		</xsl:when>
		<xsl:when test="per:int">Integer</xsl:when>
		<xsl:when test="per:long">Long</xsl:when>
		<xsl:when test="per:float">Float</xsl:when>
		<xsl:when test="per:double">Double</xsl:when>
		<xsl:when test="per:boolean">Boolean</xsl:when>
		<xsl:when test="per:date">Date</xsl:when>
		<xsl:when test="per:time">Time</xsl:when>
		<xsl:when test="per:datetime">Date time</xsl:when>
		<xsl:when test="per:timestamp">Timestamp</xsl:when>
		<xsl:when test="per:enum">
			<xsl:text>Enum [</xsl:text>
			<xsl:copy-of select="svc:link('entity',per:enum/@name)" />
			<xsl:text>]</xsl:text>
		</xsl:when>
		<xsl:when test="per:code-table">
			<xsl:text>Code table [</xsl:text>
			<xsl:copy-of select="svc:link('entity',per:code-table/@name)" />
			<xsl:text>]</xsl:text>
		</xsl:when>
		<xsl:when test="per:text">Text</xsl:when>
		<xsl:when test="per:binary">Binary</xsl:when>
		<xsl:otherwise>
			<xsl:message terminate="yes">
Unknown field type on <xsl:value-of select="concat(../@name,'.',@name)" />
			</xsl:message>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:function name="svc:link" as="node()*">
<xsl:param name="type" as="xs:string" />
<xsl:param name="name" as="xs:string" />
	<xsl:choose>
	<xsl:when test="$type='entity' and empty(index-of(distinct-values(
		$merged-service-file//svc:object/@entity | 
		$merged-service-file//svc:enum-def/@entity),$name))">
		<xsl:value-of select="$name" />
	</xsl:when>
	<xsl:otherwise>
		<a href="#{$type}_{$name}"><xsl:value-of select="$name" /></a>
	</xsl:otherwise>
	</xsl:choose>
</xsl:function>

<xsl:template match="svc:object | svc:enum-def">
	<xsl:variable name="full-type" select="svc:service-object-full-type(.)" />
	<xsl:variable name="name" select="seaf:type-class($full-type)" />
	<xsl:variable name="is-criteria" select="@processing-type='criteria-based'" />
	<xsl:text>
	</xsl:text>
	<a name="object_{$name}"><h2>Object: <xsl:value-of select="$name" /></h2></a>
	<xsl:text>
	</xsl:text>
	<span class="label">Module:</span><xsl:text> </xsl:text>
		<xsl:copy-of select="svc:link('module',ancestor::seaf:module/@name)" /><br />
	<xsl:text>
	</xsl:text>
	<span class="label">Type:</span><xsl:text> </xsl:text>
		<xsl:call-template name="object-type-template" /><br />
	<xsl:text>
	</xsl:text>
	<span class="label">Base entity:</span><xsl:text> </xsl:text>
		<xsl:copy-of select="svc:link('entity',@entity)" /><br />
	<xsl:text>
	</xsl:text>
	<span class="label">Description:</span><xsl:text> </xsl:text>
<xsl:if test="local-name()='object'">
	<xsl:variable name="svc-object" select="." />
	<xsl:call-template name="svc-descriptions-template" /><br />
	<xsl:variable name="object-fields" select="svc:object-fields(.,$merged-persistence-file)" />
	<xsl:text>
	</xsl:text>
	<span class="label">Fields:</span>
	<table>
		<tr><td>Name</td><td>Status</td><td>Type</td><td>Description</td></tr>
	<xsl:for-each select="$object-fields[not(@status='criteria-only') or $is-criteria]">
		<xsl:call-template name="field-template">
			<xsl:with-param name="svc-object" select="$svc-object" />
		</xsl:call-template>
	</xsl:for-each>
	<xsl:for-each select="svc:relation">
		<xsl:variable name="per-relation" 
			select="svc:per-relation(.,..,$merged-persistence-file)" />
		<xsl:variable name="per-relation-side" 
			select="svc:per-relation-side(.,..,$per-relation)" />
		<xsl:variable name="related-object-full-type" 
			select="svc:ref-service-object-full-type(.,$objects)" />
		<xsl:variable name="status" select="svc:field-status(.,$svc-object)" />
		<xsl:variable name="side-label" 
			select="svc:per-relation-label($per-relation-side)" />
		<xsl:variable name="is-array" 
			select="$per-relation/@type='one-to-many' and $side-label='detail'" />
		<tr>
			<td><xsl:value-of 
				select="per:relation-property-name($per-relation-side,$per-relation)"/></td>
			<td title="{per:status-hint($status)}">
				<xsl:value-of select="$status" />
			</td>
			<td>
				<xsl:if test="$is-array">Array [</xsl:if>
				<xsl:copy-of select="svc:link('object',
					seaf:type-class($related-object-full-type))"/>
				<xsl:if test="$is-array">]</xsl:if>
			</td>
			<td>Relation to <xsl:value-of select="$side-label" /> service object.</td>
		</tr>
	</xsl:for-each>
	</table>
	<xsl:variable name="omitted-object-fields" 
		select="svc:omitted-object-fields(.,$merged-persistence-file)" />
	<xsl:if test="exists($omitted-object-fields)">
		<xsl:variable name="label">
		<xsl:choose>
		<xsl:when test="svc:include">Not included</xsl:when>
		<xsl:otherwise>Excluded</xsl:otherwise>
		</xsl:choose>
		</xsl:variable>
	<xsl:text>
	</xsl:text>
	<span class="label"><xsl:value-of select="$label" /> fields:</span>
		<xsl:for-each select="$omitted-object-fields">
			<xsl:text> </xsl:text>
			<xsl:value-of select="."/>
			<xsl:if test="not(position()=last())"><xsl:text>,</xsl:text></xsl:if>
		</xsl:for-each>
	</xsl:if>
	<xsl:if test="svc:sort">
		<div><span class="label">Sort enabled fields: </span>
		<xsl:for-each select="svc:sort/svc:field/@ref">
			<xsl:value-of select="." />
			<xsl:if test="not(position()=last())">, </xsl:if>
		</xsl:for-each>
		</div>
	</xsl:if>
	<xsl:variable name="ref-objects" select="$merged-service-file//svc:service-layer/svc:object
		[svc:relation/svc:ref-service-object-full-type(.,$objects)=$full-type]" />
	<xsl:if test="$ref-objects">
		<div><span class="label">References in service objects: </span>
		<xsl:for-each select="$ref-objects">
			<xsl:variable name="ref-full-type" select="svc:service-object-full-type(.)" />
			<xsl:copy-of select="svc:link('object',seaf:type-class($ref-full-type))"/>
			<xsl:if test="not(position()=last())">, </xsl:if>
		</xsl:for-each>
		</div>
	</xsl:if>
	<xsl:variable name="ref-ops" select="$merged-service-file//svc:service-layer//svc:operation
		[(svc:request-object | svc:response-object | svc:response-sequence)/
			svc:ref-service-object-full-type(.,$objects)=$full-type]" />
	<xsl:if test="$ref-ops">
		<div><span class="label">References in operations: </span>
		<xsl:for-each select="$ref-ops">
			<xsl:copy-of select="svc:link('operation',@name)" />
			<xsl:if test="not(position()=last())">, </xsl:if>
		</xsl:for-each>
		</div>
	</xsl:if>
</xsl:if>
<xsl:if test="local-name()='enum-def'">
	<xsl:variable name="full-type" select="svc:enum-full-type(.)" />
	<xsl:variable name="persistence-enum-def" 
		select="svc:persistence-enum-ref(@entity,$merged-persistence-file)" />
	<xsl:for-each select="$persistence-enum-def">
		<xsl:call-template name="per-descriptions-template" /><br />
	</xsl:for-each>
</xsl:if>
</xsl:template>

<xsl:template name="object-type-template">
	<xsl:choose>
		<xsl:when test="local-name()='enum-def'">Enumeration</xsl:when>
		<xsl:when test="@processing-type='criteria-based'">Criteria object</xsl:when>
		<xsl:when test="@processing-type='non-validating'">Non-validating object</xsl:when>
		<xsl:otherwise>Object</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="field-template">
<xsl:param name="svc-object" as="node()" />
	<xsl:variable name="field-name" select="@name" />
	<xsl:variable name="svc-field" select="$svc-object//svc:field[@ref=$field-name]" />
	<xsl:variable name="status" select="svc:field-status($svc-field,$svc-object)" />
	<tr>
		<td><xsl:value-of select="@name"/></td>
		<td title="{per:status-hint($status)}">
			<xsl:value-of select="$status" />
		</td>
		<td>
			<xsl:call-template name="object-field-type-template">
				<xsl:with-param name="svc-field" select="$svc-field" />
				<xsl:with-param name="svc-object" select="$svc-object" />
			</xsl:call-template>
		</td>
		<td><xsl:call-template name="per-descriptions-template" /></td>
	</tr>
</xsl:template>

<xsl:function name="svc:field-status" as="xs:string">
<xsl:param name="svc-field" />
<xsl:param name="svc-object" as="node()" />
	<xsl:variable name="is-non-validating" 
		select="$svc-object/@processing-type='non-validating' 
			or $svc-object/@processing-type='criteria-based'" />
	<xsl:choose>
		<xsl:when test="$is-non-validating"><xsl:text></xsl:text></xsl:when>
		<xsl:when test="$svc-field/@optional">O</xsl:when>
		<xsl:otherwise>M</xsl:otherwise>
	</xsl:choose>
</xsl:function>

<xsl:template name="object-field-type-template">
<xsl:param name="svc-field" />
<xsl:param name="svc-object" as="node()" />
	<xsl:variable name="is-non-validating" 
		select="$svc-object/@processing-type='non-validating' 
			or $svc-object/@processing-type='criteria-based'" />
	<xsl:variable name="per-field" 
		select="svc:expand-code-table-field(.,$merged-persistence-file)" />
	<xsl:choose>
		<xsl:when test="$per-field/per:string">
			<xsl:text>String</xsl:text>
			<xsl:if test="not($is-non-validating)">
				<xsl:text> [</xsl:text>
				<xsl:if test="svc:field-has-size-min($svc-field)">
					<xsl:value-of select="svc:field-size-min($svc-field)" /> - <xsl:text />
				</xsl:if>
				<xsl:value-of select="svc:field-size-max($svc-field,$per-field)" />
				<xsl:text>]</xsl:text>
			</xsl:if>
		</xsl:when>
		<xsl:when test="per:int">
			<xsl:text>Integer</xsl:text>
			<xsl:value-of select="svc:field-min-max($svc-field,$is-non-validating)" />
		</xsl:when>
		<xsl:when test="per:long">
			<xsl:text>Long</xsl:text>
			<xsl:value-of select="svc:field-min-max($svc-field,$is-non-validating)" />
		</xsl:when>
		<xsl:when test="per:float">Float</xsl:when>
		<xsl:when test="per:double">Double</xsl:when>
		<xsl:when test="per:boolean">Boolean</xsl:when>
		<xsl:when test="per:date">Date</xsl:when>
		<xsl:when test="per:time">Time</xsl:when>
		<xsl:when test="per:datetime">Date time</xsl:when>
		<xsl:when test="per:timestamp">Timestamp</xsl:when>
		<xsl:when test="per:enum">
			<xsl:text>Enum [</xsl:text>
			<xsl:variable name="enum-type" 
				select="svc:enum-ref-full-type(per:enum/@name,$merged-service-file)" />
			<xsl:copy-of select="svc:link('object',seaf:type-class($enum-type))" />
			<xsl:text>]</xsl:text>
		</xsl:when>
		<xsl:when test="per:text">String</xsl:when>
		<xsl:when test="per:binary">byte[]</xsl:when>
		<xsl:otherwise>
			<xsl:message terminate="yes">
Unknown field type on <xsl:value-of select="concat($svc-object/@entity,'.',@name)" />
			</xsl:message>
		</xsl:otherwise>
	</xsl:choose>
	<xsl:if test="per:code-table">
		<xsl:text> (code table </xsl:text>
		<xsl:copy-of select="svc:link('entity',per:code-table/@name)" />
		<xsl:text>)</xsl:text>
	</xsl:if>
</xsl:template>

<xsl:function name="svc:field-min-max" as="xs:string*">
<xsl:param name="svc-field" />
<xsl:param name="is-non-validating" />
	<xsl:if test="not($is-non-validating) and ($svc-field/@min or $svc-field/@max)">
		<xsl:value-of select="concat(' [',$svc-field/@min,'..',$svc-field/@max,']')" />
	</xsl:if>
</xsl:function>

</xsl:stylesheet>
