<stylesheet xmlns="http://www.w3.org/1999/XSL/Transform"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:seaf="http://seaf.sf.net/seaf"
	xmlns:per="http://seaf.sf.net/seaf-persistence"
	xmlns:svc="http://seaf.sf.net/seaf-service"
	version="2.0">
<import href="generator/seaf-persistence/functions/SeafPersistenceBusinessFunctions.xsl" />

<!-- Full types -->

<function name="svc:full-type-internal" as="xs:string">
	<param name="element" as="node()" />
	<param name="name" as="xs:string" />
	<param name="package-insert" as="xs:string" />
	<param name="suffix" as="xs:string" />
	<value-of select="concat($element/ancestor::seaf:module/@package,
			'.service',$package-insert,'.',$name,$suffix)" />
</function>

<function name="svc:service-object-type" as="xs:string">
	<param name="type" as="xs:string*" />
	<choose>
		<when test="$type='default'"><text></text></when>
		<otherwise><value-of select="$type" /></otherwise>
	</choose>
</function>

<function name="svc:service-object-full-type" as="xs:string">
	<param name="object" as="node()" />
	<variable name="suffix" select="concat('SO',svc:service-object-type($object/@type))" />
	<value-of select="svc:full-type-internal($object,$object/@entity,'.object',$suffix)" />
</function>

<function name="svc:service-interface-full-type" as="xs:string">
	<param name="service" as="node()" />
	<value-of select="svc:full-type-internal($service,$service/@interface,'','')" />
</function>

<function name="svc:controller-interface-full-type" as="xs:string">
	<param name="controller" as="node()" />
	<param name="service" as="node()" />
	<variable name="suffix" select="concat($service/@interface,'Controller')" />
	<value-of select="svc:full-type-internal($service,$controller/@entity,
		'.controller',$suffix)" />
</function>

<function name="svc:enum-full-type" as="xs:string">
	<param name="enum-def" as="node()" />
	<value-of select="svc:full-type-internal($enum-def,$enum-def/@entity,'.object','SO')" />
</function>

<!-- Full ref types -->

<function name="svc:ref-service-object-full-type" as="xs:string">
	<param name="ref" as="node()" />
	<param name="objects" as="node()*" />
	<variable name="ref-entity" select="$ref/@entity" />
	<variable name="ref-type" select="$ref/@type" />
	<variable name="ref-object" select="$objects[@entity=$ref-entity and @type=$ref-type]" />
	<if test="count($ref-object) != 1">
		<message select="'Error locating referenced service object: exactly 1 expected'"/>
		<message select="concat(' reference to = ',$ref-entity,' ',$ref-type)" />
		<message terminate="yes" 
			select="concat('  - referenced service objects found = ',count($ref-object))" />
	</if>
	<value-of select="svc:service-object-full-type($ref-object)" />
</function>

<function name="svc:ref-service-object" as="node()">
	<param name="ref" as="node()" />
	<param name="objects" as="node()*" />
	<variable name="ref-entity" select="$ref/@entity" />
	<variable name="ref-type" select="$ref/@type" />
	<variable name="ref-object" select="$objects[@entity=$ref-entity and @type=$ref-type]" />
	<if test="count($ref-object) != 1">
		<message select="'Error locating referenced service object: exactly 1 expected'"/>
		<message select="concat(' reference to = ',$ref-entity,' ',$ref-type)" />
		<message terminate="yes" 
			select="concat('  - referenced service objects found = ',count($ref-object))" />
	</if>
	<copy-of select="$ref-object" />
</function>

<function name="svc:enum-ref-full-type" as="xs:string">
	<param name="entity-name" as="xs:string" />
	<param name="merged-service-file" as="node()*" />
	<variable name="enum-def" select="$merged-service-file//svc:enum-def[@entity=$entity-name]" />
	<if test="count($enum-def) != 1">
		<message select="'Error locating service enumeration definition: exactly 1 expected'"/>
		<message select="concat(' reference to = ',$entity-name)" />
		<message terminate="yes" 
			select="concat('  - referenced enumerations found = ',count($enum-def))" />
	</if>
	<value-of select="svc:enum-full-type($enum-def)" />
</function>

<function name="svc:persistence-enum-ref" as="node()">
	<param name="enum-name" as="xs:string" />
	<param name="merged-persistence-file" as="node()*" />
	<variable name="enum-def" select="$merged-persistence-file//per:enum-def[@name=$enum-name]" />
	<if test="count($enum-def) != 1">
		<message select="'Error locating persistence enumeration definition: exactly 1 expected'"/>
		<message select="concat(' reference to = ',$enum-name)" />
		<message terminate="yes" 
			select="concat('  - referenced enumerations found = ',count($enum-def))" />
	</if>
	<copy-of select="$enum-def" />
</function>

<function name="svc:object-fields" as="node()*">
	<param name="object" as="node()" />
	<param name="merged-persistence-file" as="node()*" />
	<variable name="entity" select="per:entity-ref($object/@entity,$merged-persistence-file)" />
	<variable name="all-entity-fields" select="per:entity-fields($entity,$merged-persistence-file)" />
	<variable name="included-field-names" select="$object/svc:include/svc:field/@ref" />
	<variable name="included-entity-fields" select="$all-entity-fields[exists(index-of($included-field-names,@name))]" />
	<variable name="excluded-field-names" select="$object/svc:exclude/svc:field/@ref" />
	<variable name="not-excluded-entity-fields" select="$all-entity-fields[empty(index-of($excluded-field-names,@name))]" />
	<variable name="object-fields" 
		select="if ($object/svc:include) then $included-entity-fields else 
			(if ($object/svc:exclude) then $not-excluded-entity-fields else $all-entity-fields)" />
	<variable name="invalid-field-refs" select="$included-field-names[empty(index-of($object-fields/@name,.))]" />
	<if test="count($invalid-field-refs)>0">
		<message select="'Error resolving reference to entity field'"/>
		<message select="concat(' reference to entity ',$object/@entity,
			', fields: ',string-join($invalid-field-refs,' '))" />
		<message terminate="yes" 
			select="'  - no such field(s) found on the referenced entity'" />
	</if>
	<copy-of select="$object-fields" />
</function>

<function name="svc:omitted-object-fields" as="xs:string*">
	<param name="object" as="node()" />
	<param name="merged-persistence-file" as="node()*" />
	<variable name="entity" select="per:entity-ref($object/@entity,$merged-persistence-file)" />
	<variable name="all-entity-fields" select="per:entity-fields($entity,$merged-persistence-file)" />
	<variable name="included-field-names" select="$object/svc:include/svc:field/@ref" />
	<variable name="not-included-entity-fields" select="$all-entity-fields[empty(index-of($included-field-names,@name))]" />
	<variable name="excluded-field-names" select="$object/svc:exclude/svc:field/@ref" />
	<variable name="excluded-entity-fields" select="$all-entity-fields[exists(index-of($excluded-field-names,@name))]" />
	<variable name="omitted-object-fields" 
		select="if ($object/svc:include) then $not-included-entity-fields else 
			(if ($object/svc:exclude) then $excluded-entity-fields else ())" />
	<copy-of select="$omitted-object-fields/@name" />
</function>

<function name="svc:per-relation" as="node()">
	<param name="svc-relation" as="node()" />
	<param name="object" as="node()" />
	<param name="merged-persistence-file" as="node()*" />
	<variable name="relations" select="$merged-persistence-file//per:relation" />
	<variable name="per-relation-side" select="
		$relations/per:master[
			@entity-name=$object/@entity 
				and ../per:detail/@entity-name=$svc-relation/@entity
				and (empty($svc-relation/@field-name) or 
					(../per:detail/@field-name=$svc-relation/@field-name))]
		| $relations/per:detail[
			@entity-name=$object/@entity 
				and ../per:master/@entity-name=$svc-relation/@entity
				and (empty($svc-relation/@field-name) or 
					(../per:master/@field-name=$svc-relation/@field-name))]" />
	<if test="count($per-relation-side) != 1">
		<message select="'Error locating underlying persistence relation: exactly 1 expected'"/>
		<message select="concat(' object = ',$object/@entity,' ',$object/@type)" />
		<message select="concat(' relation to = ',$svc-relation/@entity,' ',
			$svc-relation/@type,' ',$svc-relation/field-name)" />
		<message terminate="yes" 
			select="concat('  - persistence relations found = ',count($per-relation-side))" />
	</if>
	<copy-of select="$per-relation-side/.." />
</function>

<function name="svc:per-relation-side" as="node()">
	<param name="svc-relation" as="node()" />
	<param name="object" as="node()" />
	<param name="per-relation" as="node()" />
	<variable name="per-relation-side" select="
		$per-relation/per:master[
			@entity-name=$object/@entity 
				and (empty($svc-relation/@field-name) or 
					(../per:detail/@field-name=$svc-relation/@field-name))]
		| $per-relation/per:detail[
			@entity-name=$object/@entity 
				and (empty($svc-relation/@field-name) or 
					(../per:master/@field-name=$svc-relation/@field-name))]" />
	<copy-of select="$per-relation-side" />
</function>

<function name="svc:per-relation-label" as="xs:string">
	<param name="per-relation-side" as="node()" />
	<choose>
	<when test="local-name($per-relation-side)='master'">detail</when>
	<otherwise>master</otherwise>
	</choose>
</function>

<function name="svc:field-has-size-min" as="xs:boolean">
	<param name="svc-field" as="node()*" />
	<value-of select="not($svc-field/@optional) or ($svc-field/@size-min)" />
</function>

<function name="svc:field-size-min" as="xs:string">
	<param name="svc-field" as="node()*" />
	<choose>
	<when test="$svc-field/@size-min">
		<value-of select="$svc-field/@size-min" />
	</when>
	<otherwise>1</otherwise>
	</choose>
</function>

<function name="svc:field-size-max" as="xs:string">
	<param name="svc-field" as="node()*" />
	<param name="per-field" as="node()" />
	<choose>
	<when test="$svc-field/@size-max">
		<value-of select="$svc-field/@size-max" />
	</when>
	<otherwise>
		<value-of select="$per-field/per:string/@length" />
	</otherwise>
	</choose>
</function>

<function name="svc:expand-code-table-field" as="node()">
	<param name="svc-field" as="node()" />
	<param name="merged-persistence-file" as="node()*" />
	<variable name="per-field">
		<choose>
		<when test="$svc-field/per:code-table">
			<copy-of select="per:code-table-code-field(
				$svc-field/per:code-table/@name,$merged-persistence-file)" />
		</when>
		<otherwise>
			<copy-of select="$svc-field" />
		</otherwise>
		</choose>
	</variable>
	<copy-of select="$per-field/per:field" />
</function>

<function name="svc:method-signature" as="xs:string">
	<param name="operation" as="node()" />
	<param name="method-name" as="xs:string" />
	<param name="request" as="xs:string" />
	<param name="response" as="xs:string" />
	<param name="include-roles" as="xs:boolean" />
	<variable name="result">
<text />

	/**<text />
	<if test="$operation/svc:description">
	 * <value-of select="$operation/svc:description" />
	 	<if test="$operation/svc:description-list">
	 * &lt;p><text />
	 	</if>
	</if>
	<apply-templates select="$operation/svc:description-list" />
	 */<text />
	<if test="$include-roles and $operation/svc:role">
	@javax.annotation.security.RolesAllowed({ <text />
		<for-each select="$operation/svc:role">
			<text>"</text>
			<value-of select="@name" />
			<text>"</text>
			<if test="not(position()=last())">, </if>
		</for-each>
	<text> })</text>
	</if>
	<text>
	</text>
	<value-of select="$response" />
	<text> </text>
	<value-of select="$method-name" />(<value-of select="$request" />
	<if test="$operation/svc:response-sequence/@paged='true'">
		<if test="$operation/svc:request-object">,
			<text />
		</if>
		<text>net.sf.seaf.common.Page page</text>
	</if>
	<text />)<text />
	<if test="$operation/svc:throws"> throws</if>
	<for-each select="$operation/svc:throws">
		<text>
			</text>
		<value-of select="@class" />
		<if test="not(position()=last())">,</if>
	</for-each>
	<text />;<text />
	</variable>
	<value-of select="$result" />
</function>

</stylesheet>
